"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _FileSystemUtilities = require("./FileSystemUtilities");

var _FileSystemUtilities2 = _interopRequireDefault(_FileSystemUtilities);

var _PackageUtilities = require("./PackageUtilities");

var _PackageUtilities2 = _interopRequireDefault(_PackageUtilities);

var _ExitHandler = require("./ExitHandler");

var _ExitHandler2 = _interopRequireDefault(_ExitHandler);

var _progressBar = require("./progressBar");

var _progressBar2 = _interopRequireDefault(_progressBar);

var _Repository = require("./Repository");

var _Repository2 = _interopRequireDefault(_Repository);

var _logger = require("./logger");

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Command = function () {
  function Command(input, flags) {
    _classCallCheck(this, Command);

    this.input = input;
    this.flags = flags;

    this.lernaVersion = require("../package.json").version;
    this.repository = new _Repository2.default();
    this.progressBar = _progressBar2.default;
    this.logger = _logger2.default;
  }

  _createClass(Command, [{
    key: "run",
    value: function run() {
      this.runPreparations();
      this.runCommand();
    }
  }, {
    key: "runPreparations",
    value: function runPreparations() {
      this.logger.info("Lerna v" + this.lernaVersion);

      if (this.flags.independent) {
        this.logger.info("Independent Versioning Mode");
      }

      if (this.flags.canary) {
        this.logger.info("Publishing canary build");
      }

      if (!_FileSystemUtilities2.default.existsSync(this.repository.packagesLocation)) {
        this.logger.error("`packages/` directory does not exist, have you run `lerna init`?");
        this._complete(1);
        return;
      }

      if (!_FileSystemUtilities2.default.existsSync(this.repository.packageJsonLocation)) {
        this.logger.error("`package.json` does not exist, have you run `lerna init`?");
        this._complete(1);
        return;
      }

      if (!this.flags.independent && !_FileSystemUtilities2.default.existsSync(this.repository.versionLocation)) {
        this.logger.error("`VERSION` does not exist, have you run `lerna init`? Or maybe you meant to run this with `--independent` or `-i`?");
        this._complete(1);
        return;
      }

      try {
        this.packages = _PackageUtilities2.default.getPackages(this.repository.packagesLocation);
        this.packageGraph = _PackageUtilities2.default.getPackageGraph(this.packages);
      } catch (err) {
        this.logger.error("Errored while collecting packages and package graph", err);
        this._complete(1);
        throw err;
      }
    }
  }, {
    key: "runCommand",
    value: function runCommand(callback) {
      var _this = this;

      this._attempt("initialize", function () {
        _this._attempt("execute", function () {
          _this._complete(null, 0, callback);
        }, callback);
      }, callback);
    }
  }, {
    key: "_attempt",
    value: function _attempt(method, next, callback) {
      var _this2 = this;

      var methodName = this.constructor.name + "." + method;

      try {
        this.logger.debug("Attempting running " + methodName);

        this[method](function (err, completed) {
          if (err) {
            _this2.logger.error("Errored while running " + methodName, err);
            _this2._complete(err, 1, callback);
          } else if (!completed) {
            _this2.logger.debug("Exited early while running " + methodName);
            _this2._complete(null, 1, callback);
          } else {
            _this2.logger.debug("Successfully ran " + methodName);
            next();
          }
        });
      } catch (err) {
        this.logger.error("Errored while running " + methodName, err);
        this._complete(err, 1, callback);
      }
    }
  }, {
    key: "_complete",
    value: function _complete(err, code, callback) {
      if (code !== 0) {
        var exitHandler = new _ExitHandler2.default();
        exitHandler.writeLogs();
      }

      if (callback) {
        callback(err, code);
      }

      if (process.env.NODE_ENV !== "test") {
        process.exit(code);
      }
    }
  }, {
    key: "initialize",
    value: function initialize() {
      throw new Error("command.initialize() needs to be implemented.");
    }
  }, {
    key: "execute",
    value: function execute() {
      throw new Error("command.execute() needs to be implemented.");
    }
  }]);

  return Command;
}();

exports.default = Command;